<cfcomponent displayname="StateBuilderSES" extends="ModelGlue.unity.statebuilder.StateBuilder" output="false" hint="I am the state builder for SES URLs.">
<!--- 

	Modified StateBuilder by: Brian Ghidinelli 
							  http://www.ghidinelli.com/
							  brian@vfive.com
	
	This statebuilder extends the default ModelGlue:Unity to provide SES URLs as an actionpack.
	I created it following advice found in one of Joe's blog posts here:
	
	  http://www.firemoss.com/blog/index.cfm/2006/8/21/No-SES-URLs-in-ModelGlueUnity
	
	And some code from Sean Corfield's blog here:
	
	  http://corfield.org/blog/index.cfm/do/blog.search/search/humbug
	
	The only modification in this file from the stock is line 64 in the definition of "myself".

	--->

<cffunction name="init" returntype="any" output="false">
	<cfargument name="parsePathInfo" type="boolean" default="true" />
	<cfset variables.parsePathInfo = arguments.parsePathInfo />
	<cfset super.init() />
	<cfreturn this />
</cffunction>

<cffunction name="createStateContainer" output="false">
	<cfargument name="collection" />
	
	<cfset var stateContainers = "" />
	<cfset var state = "" />
	<cfset var eventValueName = getConfigSetting("eventValue") />
	<!--- remove the filename itself from path_info if it comes through --->
	<cfset var pathInfo = reReplaceNoCase(trim(cgi.path_info), '.+\.cfm/? *', '') />
	<cfset var ii = 1 />
	<cfset var lastKey = "" />
	<cfset var value = "" />
	
	<!--- We need session for redirect forwarding and async requests --->
	<cfif variables._framework.getUseSession()>
		<cfparam name="session._modelglue" default="#structNew()#">
		<cfparam name="session._ModelGlue.asyncRequests" default="#structNew()#">
		<cfparam name="session._ModelGlue.forwardedStateContainer" default="#structNew()#">
	</cfif>
	
	<!--- import the URL scope from CGI.PATH_INFO --->
	<cfif variables.parsePathInfo AND len(pathInfo)>
		<cfloop from="1" to="#listLen(pathInfo, '/=&')#" index="ii">
			<cfset value = listGetAt(pathInfo, ii, '/=&') />
			<cfif NOT ii MOD 2>
				<cfset URL[lastKey] = value />
			<cfelse>
				<cfset lastKey = value />
			</cfif>
		</cfloop>
		<!--- did we end with a dangler? --->	
		<cfif (ii - 1) MOD 2 IS 1>
			<cfset URL[lastKey] = "" />
		</cfif>
	</cfif>
	
	<!--- 
	<cfset isArg = true />
	<cfloop list="#pathInfo#" index="av" delimiters="/=&">
	   <cfif isArg>
	      <cfset argName = av />
	   <cfelse>
	      <cfset URL[argName] = av />
	   </cfif>
	   <cfset isArg = NOT isArg />
	</cfloop>
	--->

	<!--- Create our viewstate --->
	<cfset stateContainers = arrayNew(1) />
	<cfif variables._framework.getUseSession() and structCount(session._ModelGlue.forwardedStateContainer) and structKeyExists(url, eventValueName)>
		<cfset session._ModelGlue.forwardedStateContainer.setValue(eventValueName, url[eventValueName]) />
		<cfset arrayAppend(stateContainers, session._ModelGlue.forwardedStateContainer.getAll()) /> 
	</cfif>
	<cfif getConfigSetting("statePrecedence") eq "url">
		<cfset arrayAppend(stateContainers, url) />
		<cfset arrayAppend(stateContainers, form) />
	<cfelse>
		<cfset arrayAppend(stateContainers, form) />
		<cfset arrayAppend(stateContainers, url) />
	</cfif>
	
	<cfset state = arguments.collection.init(stateContainers) />

	<!--- Default event --->
	<cfif not len(state.getValue(eventValueName))>
		<cfset state.setValue(eventValueName, getConfigSetting("defaultEvent")) />
	</cfif>

	<!--- "Myself" --->
	<cfset state.setValue("self", getConfigSetting("defaultTemplate")) />
	<cfset state.setValue("eventValue", eventValueName) />
	<cfset state.setValue("myself", getConfigSetting("defaultTemplate") & "/" & eventValueName & "/") />
</cffunction>

</cfcomponent>